@@ -412,6 +412,11 @@ class SyncEnv(EnvironmentBase):
             base_url = self.urls.api
         elif self.urls and self.urls.root:
             base_url = f"{self.urls.root.rstrip('/')}/raw"
+        elif self._manager_url_override and self._manager_url_override != "local://":
+            # URL mode: strip /api/v1/env suffix to get root URL
+            base_url = self._manager_url_override.rstrip('/')
+            if base_url.endswith('/api/v1/env'):
+                base_url = base_url[:-len('/api/v1/env')]
         else:
             raise ValueError("No API URL configured for this environment")
         return self.instance.api(name, base_url)
@@ -1276,6 +1281,8 @@ class Fleet:
         prompt: Optional[str] = None,
         verifier_code: Optional[str] = None,
         metadata: Optional[Dict[str, Any]] = None,
+        writer_metadata: Optional[Dict[str, Any]] = None,
+        qa_metadata: Optional[Dict[str, Any]] = None,
     ) -> TaskResponse:
         """Update an existing task.
 
@@ -1284,11 +1291,19 @@ class Fleet:
             prompt: New prompt text for the task (optional)
             verifier_code: Python code for task verification (optional)
             metadata: Additional metadata for the task (optional)
+            writer_metadata: Metadata filled by task writer (optional)
+            qa_metadata: Metadata filled by QA reviewer (optional)
 
         Returns:
             TaskResponse containing the updated task details
         """
-        payload = TaskUpdateRequest(prompt=prompt, verifier_code=verifier_code, metadata=metadata)
+        payload = TaskUpdateRequest(
+            prompt=prompt,
+            verifier_code=verifier_code,
+            metadata=metadata,
+            writer_metadata=writer_metadata,
+            qa_metadata=qa_metadata,
+        )
         response = self.client.request(
             "PUT", f"/v1/tasks/{task_key}", json=payload.model_dump(exclude_none=True)
         )
